@@ -26,9 +26,6 @@
 /* Define if you want to use MySQL */
 #undef HAVE_MYSQL
 
-/* Defined if MySQL thread safe library is present */
-#undef HAVE_THREAD_SAFE_MYSQL
-
 /* Define if you want to use embedded MySQL */
 #undef HAVE_EMBEDDED_MYSQL
 
@@ -52,9 +49,6 @@
 /* Define if you have GCC */
 #undef HAVE_GCC
 
-/* Define if you have the Andrew File System.  */
-#undef AFS
-
 /* Define If you want find -nouser and -nogroup to make tables of
    used UIDs and GIDs at startup instead of using getpwuid or
    getgrgid when needed.  Speeds up -nouser and -nogroup unless you
@@ -120,9 +114,6 @@
 /* Define if you have libacl */
 #undef HAVE_ACL
 
-/* Define if you have AFS acls */
-#undef HAVE_AFS_ACL
-
 /* General libs */
 #undef LIBS
 
@@ -207,4 +198,3 @@
 
 /* Define if language support is enabled */
 #undef ENABLE_NLS
-
