@@ -241,12 +241,16 @@ bail_out:
 
 void B_DB_MYSQL::db_close_database(JCR *jcr)
 {
-   db_end_transaction(jcr);
+   if (m_connected) {
+      db_end_transaction(jcr);
+   }
    P(mutex);
    m_ref_count--;
    Dmsg3(100, "closedb ref=%d connected=%d db=%p\n", m_ref_count, m_connected, m_db_handle);
    if (m_ref_count == 0) {
-      sql_free_result();
+      if (m_connected) {
+         sql_free_result();
+      }
       db_list->remove(this);
       if (m_connected) {
          Dmsg1(100, "close db=%p\n", m_db_handle);
@@ -256,7 +260,9 @@ void B_DB_MYSQL::db_close_database(JCR *jcr)
 //       mysql_server_end();
 #endif
       }
-      rwl_destroy(&m_lock);
+      if (rwl_is_init(&m_lock)) {
+         rwl_destroy(&m_lock);
+      }
       free_pool_memory(errmsg);
       free_pool_memory(cmd);
       free_pool_memory(cached_path);
@@ -588,6 +594,11 @@ bool B_DB_MYSQL::sql_batch_start(JCR *jcr)
                               "DeltaSeq integer)");
    db_unlock(this);
 
+   /*
+    * Keep track of the number of changes in batch mode.
+    */
+   changes = 0;
+
    return retval;
 }
 
@@ -600,6 +611,13 @@ bool B_DB_MYSQL::sql_batch_end(JCR *jcr, const char *error)
 {
    m_status = 0;
 
+   /*
+    * Flush any pending inserts.
+    */
+   if (changes) {
+      return sql_query(cmd);
+   }
+
    return true;
 }
 
@@ -624,12 +642,40 @@ bool B_DB_MYSQL::sql_batch_insert(JCR *jcr, ATTR_DBR *ar)
       digest = ar->Digest;
    }
 
-   Mmsg(cmd, "INSERT INTO batch VALUES "
-        "(%u,%s,'%s','%s','%s','%s',%u)",
-        ar->FileIndex, edit_int64(ar->JobId,ed1), esc_path,
-        esc_name, ar->attr, digest, ar->DeltaSeq);
+   /*
+    * Try to batch up multiple inserts using multi-row inserts.
+    */
+   if (changes == 0) {
+      Mmsg(cmd, "INSERT INTO batch VALUES "
+           "(%u,%s,'%s','%s','%s','%s',%u)",
+           ar->FileIndex, edit_int64(ar->JobId,ed1), esc_path,
+           esc_name, ar->attr, digest, ar->DeltaSeq);
+      changes++;
+   } else {
+      /*
+       * We use the esc_obj for temporary storage otherwise
+       * we keep on copying data.
+       */
+      Mmsg(esc_obj, ",(%u,%s,'%s','%s','%s','%s',%u)",
+           ar->FileIndex, edit_int64(ar->JobId,ed1), esc_path,
+           esc_name, ar->attr, digest, ar->DeltaSeq);
+      pm_strcat(cmd, esc_obj);
+      changes++;
+   }
 
-   return sql_query(cmd);
+   /*
+    * See if we need to flush the query buffer filled
+    * with multi-row inserts.
+    */
+   if ((changes % MYSQL_CHANGES_PER_BATCH_INSERT) == 0) {
+      if (!sql_query(cmd)) {
+         changes = 0;
+         return false;
+      } else {
+         changes = 0;
+      }
+   }
+   return true;
 }
 
 /*
