@@ -762,6 +762,7 @@ bool allow_duplicate_job(JCR *jcr)
              */
             if (cancel_me) {
               /* Zap current job */
+              jcr->setJobStatus(JS_Canceled);
               Jmsg(jcr, M_FATAL, 0, _("JobId %d already running. Duplicate job not allowed.\n"),
                  djcr->JobId);
               break;     /* get out of foreach_jcr */
@@ -796,6 +797,7 @@ bool allow_duplicate_job(JCR *jcr)
             Jmsg(jcr, M_INFO, 0, _("Cancelling duplicate JobId=%d.\n"), djcr->JobId);
             cancel_job(ua, djcr);
             bmicrosleep(0, 500000);
+            djcr->setJobStatus(JS_Canceled);
             cancel_job(ua, djcr);
             free_ua_context(ua);
             Dmsg2(800, "Cancel dup %p JobId=%d\n", djcr, djcr->JobId);
@@ -803,6 +805,7 @@ bool allow_duplicate_job(JCR *jcr)
             /*
              * Zap current job
              */
+            jcr->setJobStatus(JS_Canceled);
             Jmsg(jcr, M_FATAL, 0, _("JobId %d already running. Duplicate job not allowed.\n"),
                djcr->JobId);
             Dmsg2(800, "Cancel me %p JobId=%d\n", jcr, jcr->JobId);
@@ -1458,12 +1461,16 @@ bool run_console_command(JCR *jcr, const char *cmd)
    bool ok;
    JCR *ljcr = new_control_jcr("-RunScript-", JT_CONSOLE);
    ua = new_ua_context(ljcr);
-   /* run from runscript and check if commands are autorized */
+   /* run from runscript and check if commands are authorized */
    ua->runscript = true;
    Mmsg(ua->cmd, "%s", cmd);
    Dmsg1(100, "Console command: %s\n", ua->cmd);
    parse_ua_args(ua);
-   ok= do_a_command(ua);
+   if (ua->argc > 0 && ua->argk[0][0] == '.') {
+      ok = do_a_dot_command(ua);
+   } else {
+     ok = do_a_command(ua);
+   }
    free_ua_context(ua);
    free_jcr(ljcr);
    return ok;
