@@ -1,7 +1,7 @@
 /*
    Bacula® - The Network Backup Solution
 
-   Copyright (C) 2000-2011 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2012 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -156,7 +156,7 @@ static struct cmdstruct commands[] = {                                      /* C
  { NT_("query"),      querycmd,      _("Query catalog"),              NT_(""),              false},
  { NT_("restore"),    restore_cmd,   _("Restore files"), 
    NT_("where=</path> client=<client> storage=<storage> bootstrap=<file> "
-       "restore_job=<job>"
+       "restorejob=<job>"
        "\n\tcomment=<text> jobid=<jobid> done select all"), false},
 
  { NT_("relabel"),    relabel_cmd,   _("Relabel a tape"), 
@@ -294,7 +294,6 @@ static int add_cmd(UAContext *ua, const char *cmd)
    }
 
    memset(&pr, 0, sizeof(pr));
-   memset(&mr, 0, sizeof(mr));
 
    if (!get_pool_dbr(ua, &pr)) {
       return 1;
@@ -398,8 +397,8 @@ static int add_cmd(UAContext *ua, const char *cmd)
       bsnprintf(mr.VolumeName, sizeof(mr.VolumeName), name, i);
       mr.Slot = Slot++;
       mr.InChanger = InChanger;
-      mr.StorageId = store->StorageId;
       mr.Enabled = 1;
+      set_storageid_in_mr(store, &mr);
       Dmsg1(200, "Create Volume %s\n", mr.VolumeName);
       if (!db_create_media_record(ua->jcr, ua->db, &mr)) {
          ua->error_msg("%s", db_strerror(ua->db));
@@ -1145,9 +1144,14 @@ static int estimate_cmd(UAContext *ua, const char *cmd)
          }
       }
       if (strcasecmp(ua->argk[i], NT_("accurate")) == 0) {
-         if (!is_yesno(ua->argv[i], &accurate)) {
-            ua->error_msg(_("Invalid value for accurate. " 
-                            "It must be yes or no.\n"));
+         if (ua->argv[i]) {
+            if (!is_yesno(ua->argv[i], &accurate)) {
+               ua->error_msg(_("Invalid value for accurate. " 
+                               "It must be yes or no.\n"));
+            }
+            continue;
+         } else {
+            ua->error_msg(_("Accurate value missing.\n"));
          }
       }
    }
@@ -1188,6 +1192,7 @@ static int estimate_cmd(UAContext *ua, const char *cmd)
 
    jcr->job = job;
    jcr->setJobType(JT_BACKUP);
+   jcr->start_time = time(NULL);
    init_jcr_job_record(jcr);
 
    if (!get_or_create_client_record(jcr)) {
@@ -1206,16 +1211,6 @@ static int estimate_cmd(UAContext *ua, const char *cmd)
       return 1;
    }
 
-   if (!send_include_list(jcr)) {
-      ua->error_msg(_("Error sending include list.\n"));
-      goto bail_out;
-   }
-
-   if (!send_exclude_list(jcr)) {
-      ua->error_msg(_("Error sending exclude list.\n"));
-      goto bail_out;
-   }
-
    /* The level string change if accurate mode is enabled */
    if (accurate >= 0) {
       jcr->accurate = accurate;
@@ -1227,6 +1222,16 @@ static int estimate_cmd(UAContext *ua, const char *cmd)
       goto bail_out;
    }
 
+   if (!send_include_list(jcr)) {
+      ua->error_msg(_("Error sending include list.\n"));
+      goto bail_out;
+   }
+
+   if (!send_exclude_list(jcr)) {
+      ua->error_msg(_("Error sending exclude list.\n"));
+      goto bail_out;
+   }
+
    /*
     * If the job is in accurate mode, we send the list of
     * all files to FD.
