@@ -194,7 +194,7 @@ int insert_tree_handler(void *ctx, int num_fields, char **row)
    int32_t delta_seq;
    JobId_t JobId;
 
-   Dmsg4(100, "Path=%s%s FI=%s JobId=%s\n", row[0], row[1],
+   Dmsg4(150, "Path=%s%s FI=%s JobId=%s\n", row[0], row[1],
          row[2], row[3]);
    if (*row[1] == 0) {                 /* no filename => directory */
       if (!IsPathSeparator(*row[0])) { /* Must be Win32 directory */
@@ -210,7 +210,7 @@ int insert_tree_handler(void *ctx, int num_fields, char **row)
    JobId = str_to_int64(row[3]);
    FileIndex = str_to_int64(row[2]);
    delta_seq = str_to_int64(row[5]);
-   Dmsg5(100, "node=0x%p JobId=%s FileIndex=%s Delta=%s node.delta=%d\n",
+   Dmsg5(150, "node=0x%p JobId=%s FileIndex=%s Delta=%s node.delta=%d\n",
          node, row[3], row[2], row[5], node->delta_seq);
 
    /* TODO: check with hardlinks */
@@ -224,9 +224,12 @@ int insert_tree_handler(void *ctx, int num_fields, char **row)
             tree_remove_node(tree->root, node);
 
          } else {
-            Dmsg3(0, "Something is wrong with Delta, skipt it "
-                  "fname=%s d1=%d d2=%d\n", 
-                  row[1], node->delta_seq, delta_seq);
+            tree->ua->warning_msg(_("Something is wrong with the Delta sequence of %s, "
+                                    "skiping new parts. Current sequence is %d\n"),
+                                  row[1], node->delta_seq);
+            
+            Dmsg3(0, "Something is wrong with Delta, skip it "
+                  "fname=%s d1=%d d2=%d\n", row[1], node->delta_seq, delta_seq);
          }
          return 0;
       }
@@ -798,10 +801,6 @@ static int cdcmd(UAContext *ua, TREE_CTX *tree)
       ua->error_msg(_("Too few or too many arguments. Try using double quotes.\n"));
       return 1;
    }
-   if (!tree_node_has_child(tree->node)) {
-      ua->send_msg(_("Node %s has no children.\n"), tree->node->fname);
-      return 1;
-   }
 
    node = tree_cwd(ua->argk[1], tree->root, tree->node);
    if (!node) {
