@@ -50,7 +50,6 @@ int32_t path_max;              /* path name max length */
 #define bmalloc(x) sm_malloc(__FILE__, __LINE__, x)
 #endif
 static int our_callback(JCR *jcr, FF_PKT *ff, bool top_level);
-static bool accept_file(FF_PKT *ff);
 
 static const int fnmode = 0;
 
@@ -178,14 +177,22 @@ find_files(JCR *jcr, FF_PKT *ff, int file_save(JCR *jcr, FF_PKT *ff_pkt, bool to
    findFILESET *fileset = ff->fileset;
    if (fileset) {
       int i, j;
+      /* TODO: We probably need be move the initialization in the fileset loop,
+       * at this place flags options are "concatenated" accross Include {} blocks
+       * (not only Options{} blocks inside a Include{})
+       */
       ff->flags = 0;
-      ff->VerifyOpts[0] = 'V';
-      ff->VerifyOpts[1] = 0;
-      strcpy(ff->AccurateOpts, "Cmcs");  /* mtime+ctime+size by default */
-      strcpy(ff->BaseJobOpts, "Jspug5"); /* size+perm+user+group+chk  */
       for (i=0; i<fileset->include_list.size(); i++) {
          findINCEXE *incexe = (findINCEXE *)fileset->include_list.get(i);
          fileset->incexe = incexe;
+
+         /* Here, we reset some values between two different Include{} */
+         strcpy(ff->VerifyOpts, "V");
+         strcpy(ff->AccurateOpts, "Cmcs");  /* mtime+ctime+size by default */
+         strcpy(ff->BaseJobOpts, "Jspug5"); /* size+perm+user+group+chk  */
+         ff->plugin = NULL;
+         ff->opt_plugin = false;
+
          /*
           * By setting all options, we in effect OR the global options
           *   which is what we want.
@@ -198,9 +205,11 @@ find_files(JCR *jcr, FF_PKT *ff, int file_save(JCR *jcr, FF_PKT *ff_pkt, bool to
             ff->strip_path = fo->strip_path;
             ff->fstypes = fo->fstype;
             ff->drivetypes = fo->drivetype;
-            ff->plugin = fo->plugin; /* TODO: generate a plugin event ? */
-            ff->opt_plugin = (ff->plugin != NULL)? true : false;
-            bstrncat(ff->VerifyOpts, fo->VerifyOpts, sizeof(ff->VerifyOpts));
+            if (fo->plugin != NULL) {
+               ff->plugin = fo->plugin; /* TODO: generate a plugin event ? */
+               ff->opt_plugin = true;
+            }
+            bstrncat(ff->VerifyOpts, fo->VerifyOpts, sizeof(ff->VerifyOpts)); /* TODO: Concat or replace? */
             if (fo->AccurateOpts[0]) {
                bstrncpy(ff->AccurateOpts, fo->AccurateOpts, sizeof(ff->AccurateOpts));
             }
@@ -208,7 +217,8 @@ find_files(JCR *jcr, FF_PKT *ff, int file_save(JCR *jcr, FF_PKT *ff_pkt, bool to
                bstrncpy(ff->BaseJobOpts, fo->BaseJobOpts, sizeof(ff->BaseJobOpts));
             }
          }
-         Dmsg3(50, "Verify=<%s> Accurate=<%s> BaseJob=<%s>\n", ff->VerifyOpts, ff->AccurateOpts, ff->BaseJobOpts);
+         Dmsg4(50, "Verify=<%s> Accurate=<%s> BaseJob=<%s> flags=<%d>\n", 
+               ff->VerifyOpts, ff->AccurateOpts, ff->BaseJobOpts, ff->flags);
          dlistString *node;
          foreach_dlist(node, &incexe->name_list) {
             char *fname = node->c_str();
@@ -279,7 +289,7 @@ bool is_in_fileset(FF_PKT *ff)
 }
 
 
-static bool accept_file(FF_PKT *ff)
+bool accept_file(FF_PKT *ff)
 {
    int i, j, k;
    int fnm_flags;
