@@ -57,14 +57,14 @@ devlock *new_devlock()
    return lock;
 }
 
-int devlock::init(int priority)                            
+int devlock::init(int initial_priority)
 {
    int stat;
    devlock *rwl = this;
 
    rwl->r_active = rwl->w_active = 0;
    rwl->r_wait = rwl->w_wait = 0;
-   rwl->priority = priority;
+   rwl->priority = initial_priority;
    if ((stat = pthread_mutex_init(&rwl->mutex, NULL)) != 0) {
       return stat;
    }
