@@ -233,7 +233,7 @@ bool acquire_device_for_read(DCR *dcr)
        * If it is a tape, it checks the volume name
        */
       Dmsg1(100, "bstored: open vol=%s\n", dcr->VolumeName);
-      if (dev->open(dcr, OPEN_READ_ONLY) < 0) {
+      if (!dev->open(dcr, OPEN_READ_ONLY)) {
          if (!dev->poll) {
             Jmsg3(jcr, M_WARNING, 0, _("Read open device %s Volume \"%s\" failed: ERR=%s\n"),
                   dev->print_name(), dcr->VolumeName, dev->bstrerror());
@@ -338,11 +338,11 @@ default_path:
 
 get_out:
    dev->dlock();
+   dcr->clear_reserved();
    /* If failed and not writing plugin close device */
-   if (!ok && dev->num_writers == 0) {
+   if (!ok && dev->num_writers == 0 && dev->num_reserved() == 0) {
       generate_plugin_event(jcr, bsdEventDeviceClose, dcr);
    }
-   dcr->clear_reserved();
    /* 
     * Normally we are blocked, but in at least one error case above 
     *   we are not blocked because we unsuccessfully tried changing
@@ -539,6 +539,7 @@ bool release_device(DCR *dcr)
        *   there are no writers. It was probably reserved.
        */
       volume_unused(dcr);
+      generate_plugin_event(jcr, bsdEventDeviceClose, dcr);
    }
    Dmsg3(100, "%d writers, %d reserve, dev=%s\n", dev->num_writers, dev->num_reserved(),
          dev->print_name());
@@ -660,7 +661,7 @@ DCR *new_dcr(JCR *jcr, DCR *dcr, DEVICE *dev)
          detach_dcr_from_dev(dcr);
       }
       /* Use job spoolsize prior to device spoolsize */
-      if (jcr->spool_size) {
+      if (jcr && jcr->spool_size) {
          dcr->max_job_spool_size = jcr->spool_size;
       } else {
          dcr->max_job_spool_size = dev->device->max_job_spool_size;
