@@ -317,7 +317,7 @@ public:
    int is_tape() const { return (dev_type == B_TAPE_DEV || 
                                  dev_type == B_VTAPE_DEV); }
    int is_ftp() const { return dev_type == B_FTP_DEV; }
-   int is_file() const { return dev_type == B_FILE_DEV; }
+   int is_file() const { return (dev_type == B_FILE_DEV); }
    int is_fifo() const { return dev_type == B_FIFO_DEV; }
    int is_dvd() const  { return dev_type == B_DVD_DEV; }
    int is_vtl() const  { return dev_type == B_VTL_DEV; }
@@ -409,7 +409,7 @@ public:
    void clear_volhdr();          /* in dev.c */
    void close();                 /* in dev.c */
    void close_part(DCR *dcr);    /* in dev.c */
-   int open(DCR *dcr, int mode); /* in dev.c */
+   bool open(DCR *dcr, int mode); /* in dev.c */
    void term(void);              /* in dev.c */
    ssize_t read(void *buf, size_t len); /* in dev.c */
    ssize_t write(const void *buf, size_t len);  /* in dev.c */
@@ -485,6 +485,10 @@ inline const char *DEVICE::strerror() const { return errmsg; }
 inline const char *DEVICE::archive_name() const { return dev_name; }
 inline const char *DEVICE::print_name() const { return prt_name; }
 
+
+#define CHECK_BLOCK_NUMBERS    true
+#define NO_BLOCK_NUMBER_CHECK  false
+
 /*
  * Device Context (or Control) Record.
  *  There is one of these records for each Job that is using
@@ -544,19 +548,12 @@ public:
    VOLUME_CAT_INFO VolCatInfo;        /* Catalog info for desired volume */
 
    /* Methods */
+   void set_dev(DEVICE *ndev) { dev = ndev; }; 
    bool found_in_use() const { return m_found_in_use; };
    void set_found_in_use() { m_found_in_use = true; };
    void clear_found_in_use() { m_found_in_use = false; };
    bool is_reserved() const { return m_reserved; };
    bool is_dev_locked() { return m_dev_locked; }
-#ifdef SD_DEBUG_LOCK
-   void _dlock(const char *, int);      /* in lock.c */
-   void _dunlock(const char *, int);    /* in lock.c */
-#else
-   void dlock() { dev->dlock(); m_dev_locked = true; }
-   void dunlock() { m_dev_locked = false; dev->dunlock(); }
-#endif
-   void dblock(int why) { dev->dblock(why); }
    void setVolCatInfo(bool valid) { VolCatInfo.is_valid = valid; };
    bool haveVolCatInfo() const { return VolCatInfo.is_valid; };
    void setVolCatName(const char *name) {
@@ -565,6 +562,18 @@ public:
    };
    char *getVolCatName() { return VolCatInfo.VolCatName; };
 
+   /* Methods in lock.c */
+   void dblock(int why) { dev->dblock(why); }
+#ifdef SD_DEBUG_LOCK
+   void _dlock(const char *, int);      /* in lock.c */
+   void _dunlock(const char *, int);    /* in lock.c */
+#else
+   void dlock() { dev->dlock(); m_dev_locked = true; }
+   void dunlock() { m_dev_locked = false; dev->dunlock(); }
+#endif
+
+   /* Methods in record.c */
+   bool write_record(DEV_RECORD *rec);
 
    /* Methods in reserve.c */
    void clear_reserved();
@@ -590,6 +599,16 @@ public:
    bool do_unload();
    bool do_load(bool is_writing);
    bool is_tape_position_ok();
+
+   /* Methods in block.c */
+   bool write_block_to_device();
+   bool write_block_to_dev();
+   bool read_block_from_device(bool check_block_numbers);
+   bool read_block_from_dev(bool check_block_numbers);
+
+   /* Methods in label.c */
+   bool rewrite_volume_label(bool recycle);
+   
 };
 
 /*
