@@ -249,8 +249,10 @@ void *handle_connection_request(void *arg)
    }
 bail_out:
    generate_daemon_event(jcr, "JobEnd");
+   generate_plugin_event(jcr, bsdEventJobEnd);
    dequeue_messages(jcr);             /* send any queued messages */
    bs->signal(BNET_TERMINATE);
+   free_plugins(jcr);                 /* release instantiated plugins */
    free_jcr(jcr);
    return NULL;
 }
@@ -276,6 +278,7 @@ static bool die_cmd(JCR *jcr)
    
    Pmsg1(000, "I have been requested to die ... (%s)\n", dir->msg);
    a = djcr->JobId;   /* ref NULL pointer */
+   djcr->JobId = a;
 #endif
    return 0;
 }
@@ -313,43 +316,51 @@ static bool cancel_cmd(JCR *cjcr)
    int oldStatus;
    char Job[MAX_NAME_LENGTH];
    JCR *jcr;
+   int status;
+   const char *reason;
 
    if (sscanf(dir->msg, "cancel Job=%127s", Job) == 1) {
-      if (!(jcr=get_jcr_by_full_name(Job))) {
-         dir->fsend(_("3904 Job %s not found.\n"), Job);
-      } else {
-         oldStatus = jcr->JobStatus;
-         jcr->setJobStatus(JS_Canceled);
-         Dmsg2(800, "Cancel JobId=%d %p\n", jcr->JobId, jcr);
-         if (!jcr->authenticated && oldStatus == JS_WaitFD) {
-            pthread_cond_signal(&jcr->job_start_wait); /* wake waiting thread */
-         }
-         if (jcr->file_bsock) {
-            jcr->file_bsock->set_terminated();
-            jcr->file_bsock->set_timed_out();
-            Dmsg2(800, "Term bsock jid=%d %p\n", jcr->JobId, jcr);
-         } else {
-            /* Still waiting for FD to connect, release it */
-            pthread_cond_signal(&jcr->job_start_wait); /* wake waiting job */
-            Dmsg2(800, "Signal FD connect jid=%d %p\n", jcr->JobId, jcr);
-         }
-         /* If thread waiting on mount, wake him */
-         if (jcr->dcr && jcr->dcr->dev && jcr->dcr->dev->waiting_for_mount()) {
-            pthread_cond_broadcast(&jcr->dcr->dev->wait_next_vol);
-            Dmsg1(100, "JobId=%u broadcast wait_device_release\n", (uint32_t)jcr->JobId);
-            pthread_cond_broadcast(&wait_device_release);
-         }
-         if (jcr->read_dcr && jcr->read_dcr->dev && jcr->read_dcr->dev->waiting_for_mount()) {
-            pthread_cond_broadcast(&jcr->read_dcr->dev->wait_next_vol);
-            Dmsg1(100, "JobId=%u broadcast wait_device_release\n", (uint32_t)jcr->JobId);
-            pthread_cond_broadcast(&wait_device_release);
-         }
-         dir->fsend(_("3000 JobId=%ld Job=\"%s\" marked to be canceled.\n"), jcr->JobId, jcr->Job);
-         free_jcr(jcr);
-      }
+      status = JS_Canceled;
+      reason = "canceled";
    } else {
       dir->fsend(_("3903 Error scanning cancel command.\n"));
+      goto bail_out;
    }
+   if (!(jcr=get_jcr_by_full_name(Job))) {
+      dir->fsend(_("3904 Job %s not found.\n"), Job);
+   } else {
+      oldStatus = jcr->JobStatus;
+      jcr->setJobStatus(status);
+      Dmsg2(800, "Cancel JobId=%d %p\n", jcr->JobId, jcr);
+      if (!jcr->authenticated && oldStatus == JS_WaitFD) {
+         pthread_cond_signal(&jcr->job_start_wait); /* wake waiting thread */
+      }
+      if (jcr->file_bsock) {
+         jcr->file_bsock->set_terminated();
+         jcr->file_bsock->set_timed_out();
+         Dmsg2(800, "Term bsock jid=%d %p\n", jcr->JobId, jcr);
+      } else {
+         /* Still waiting for FD to connect, release it */
+         pthread_cond_signal(&jcr->job_start_wait); /* wake waiting job */
+         Dmsg2(800, "Signal FD connect jid=%d %p\n", jcr->JobId, jcr);
+      }
+      /* If thread waiting on mount, wake him */
+      if (jcr->dcr && jcr->dcr->dev && jcr->dcr->dev->waiting_for_mount()) {
+         pthread_cond_broadcast(&jcr->dcr->dev->wait_next_vol);
+         Dmsg1(100, "JobId=%u broadcast wait_device_release\n", (uint32_t)jcr->JobId);
+         pthread_cond_broadcast(&wait_device_release);
+      }
+      if (jcr->read_dcr && jcr->read_dcr->dev && jcr->read_dcr->dev->waiting_for_mount()) {
+         pthread_cond_broadcast(&jcr->read_dcr->dev->wait_next_vol);
+         Dmsg1(100, "JobId=%u broadcast wait_device_release\n", (uint32_t)jcr->JobId);
+         pthread_cond_broadcast(&wait_device_release);
+      }
+      dir->fsend(_("3000 JobId=%ld Job=\"%s\" marked to be %s.\n"), 
+         jcr->JobId, jcr->Job, reason);
+      free_jcr(jcr);
+   }
+
+bail_out:
    dir->signal(BNET_EOD);
    return 1;
 }
@@ -477,7 +488,7 @@ static void label_volume_if_ok(DCR *dcr, char *oldname,
    }
    /* Set old volume name for open if relabeling */
    dcr->setVolCatName(volname);
-   if (dev->open(dcr, mode) < 0) {
+   if (!dev->open(dcr, mode)) {
       dir->fsend(_("3910 Unable to open device \"%s\": ERR=%s\n"),
          dev->print_name(), dev->bstrerror());
       goto bail_out;      
@@ -513,7 +524,7 @@ static void label_volume_if_ok(DCR *dcr, char *oldname,
    case VOL_IO_ERROR:
    case VOL_NO_LABEL:
       if (!write_new_volume_label_to_dev(dcr, newname, poolname, 
-           relabel, true /* write dvd now */)) {
+              relabel, true /* write dvd now */)) {
          dir->fsend(_("3912 Failed to label Volume: ERR=%s\n"), dev->bstrerror());
          break;
       }
@@ -533,6 +544,9 @@ static void label_volume_if_ok(DCR *dcr, char *oldname,
    }
 
 bail_out:
+   if (dev->is_open() && !dev->has_cap(CAP_ALWAYSOPEN)) {
+      dev->close();
+   }
    if (!dev->is_open()) {
       dev->clear_volhdr();
    }
@@ -694,7 +708,7 @@ static bool mount_cmd(JCR *jcr)
                try_autoload_device(jcr, dcr, slot, "");
             }
             /* We freed the device, so reopen it and wake any waiting threads */
-            if (dev->open(dcr, OPEN_READ_ONLY) < 0) {
+            if (!dev->open(dcr, OPEN_READ_ONLY)) {
                dir->fsend(_("3901 Unable to open device \"%s\": ERR=%s\n"),
                   dev->print_name(), dev->bstrerror());
                if (dev->blocked() == BST_UNMOUNTED) {
@@ -752,7 +766,7 @@ static bool mount_cmd(JCR *jcr)
                              dev->print_name());
                }
             } else if (dev->is_tape()) {
-               if (dev->open(dcr, OPEN_READ_ONLY) < 0) {
+               if (!dev->open(dcr, OPEN_READ_ONLY)) {
                   dir->fsend(_("3901 Unable to open device \"%s\": ERR=%s\n"),
                      dev->print_name(), dev->bstrerror());
                   break;
@@ -766,6 +780,9 @@ static bool mount_cmd(JCR *jcr)
                                     "If this is not a blank tape, try unmounting and remounting the Volume.\n"),
                              dev->print_name());
                }
+               if (dev->is_open() && !dev->has_cap(CAP_ALWAYSOPEN)) {
+                  dev->close();
+               }
             } else if (dev->is_unmountable()) {
                if (dev->mount(1)) {
                   dir->fsend(_("3002 Device \"%s\" is mounted.\n"), dev->print_name());
@@ -1104,7 +1121,7 @@ static bool changer_cmd(JCR *jcr)
          dev = dcr->dev;
          dev->dlock();                 /* Use P to avoid indefinite block */
          if (!dev->device->changer_res) {
-            dir->fsend(_("3995 Device \"%s\" is not an autochanger.\n"), 
+            dir->fsend(_("3998 Device \"%s\" is not an autochanger.\n"), 
                dev->print_name());
          /* Under certain "safe" conditions, we can steal the lock */
          } else if (safe_cmd || !dev->is_open() || dev->can_steal_lock()) {
@@ -1180,7 +1197,7 @@ static void read_volume_label(JCR *jcr, DCR *dcr, DEVICE *dev, int Slot)
    BSOCK *dir = jcr->dir_bsock;
    bsteal_lock_t hold;
 
-   dcr->dev = dev;
+   dcr->set_dev(dev);
    steal_device_lock(dev, &hold, BST_WRITING_LABEL);
 
    if (!try_autoload_device(jcr, dcr, Slot, "")) {
