@@ -1,7 +1,7 @@
 /*
    Bacula® - The Network Backup Solution
 
-   Copyright (C) 2001-2010 Free Software Foundation Europe e.V.
+   Copyright (C) 2001-2012 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -27,7 +27,7 @@
 */
 /*
  *
- *   record.c -- tape record handling functions
+ *   record.c -- Volume (tape/disk) record handling functions
  *
  *              Kern Sibbald, April MMI
  *                added BB02 format October MMII
@@ -246,6 +246,7 @@ DEV_RECORD *new_record(void)
    rec = (DEV_RECORD *)get_memory(sizeof(DEV_RECORD));
    memset(rec, 0, sizeof(DEV_RECORD));
    rec->data = get_pool_memory(PM_MESSAGE);
+   rec->state = st_none;
    return rec;
 }
 
@@ -255,7 +256,8 @@ void empty_record(DEV_RECORD *rec)
    rec->VolSessionId = rec->VolSessionTime = 0;
    rec->FileIndex = rec->Stream = 0;
    rec->data_len = rec->remainder = 0;
-   rec->state &= ~(REC_PARTIAL_RECORD|REC_BLOCK_EMPTY|REC_NO_MATCH|REC_CONTINUATION);
+   rec->state_bits &= ~(REC_PARTIAL_RECORD|REC_BLOCK_EMPTY|REC_NO_MATCH|REC_CONTINUATION);
+   rec->state = st_none;
 }
 
 /*
@@ -273,6 +275,151 @@ void free_record(DEV_RECORD *rec)
    Dmsg0(950, "Leave free_record.\n");
 }
 
+static bool write_header_to_block(DEV_BLOCK *block, DEV_RECORD *rec)
+{
+   ser_declare;
+
+   rec->remlen = block->buf_len - block->binbuf;
+   /* Require enough room to write a full header */
+   if (rec->remlen >= WRITE_RECHDR_LENGTH) {
+      ser_begin(block->bufp, WRITE_RECHDR_LENGTH);
+      if (BLOCK_VER == 1) {
+         ser_uint32(rec->VolSessionId);
+         ser_uint32(rec->VolSessionTime);
+      } else {
+         block->VolSessionId = rec->VolSessionId;
+         block->VolSessionTime = rec->VolSessionTime;
+      }
+      ser_int32(rec->FileIndex);
+      ser_int32(rec->Stream);
+      ser_uint32(rec->data_len);
+
+      block->bufp += WRITE_RECHDR_LENGTH;
+      block->binbuf += WRITE_RECHDR_LENGTH;
+      rec->remlen -= WRITE_RECHDR_LENGTH;
+      rec->remainder = rec->data_len;
+      if (rec->FileIndex > 0) {
+         /* If data record, update what we have in this block */
+         if (block->FirstIndex == 0) {
+            block->FirstIndex = rec->FileIndex;
+         }
+         block->LastIndex = rec->FileIndex;
+      }
+   } else {
+      rec->remainder = rec->data_len + WRITE_RECHDR_LENGTH;
+      return false;
+   }
+   return true;
+}
+
+static void write_continue_header_to_block(DEV_BLOCK *block, DEV_RECORD *rec)
+{
+   ser_declare;
+
+   rec->remlen = block->buf_len - block->binbuf;
+   /*
+    * We have unwritten bytes from a previous
+    * time. Presumably we have a new buffer (possibly
+    * containing a volume label), so the new header
+    * should be able to fit in the block -- otherwise we have
+    * an error.  Note, we have to continue splitting the
+    * data record if it is longer than the block.
+    *
+    * First, write the header.
+    *
+    * Every time we write a header and it is a continuation
+    * of a previous partially written record, we store the
+    * Stream as -Stream in the record header.
+    */
+   ser_begin(block->bufp, WRITE_RECHDR_LENGTH);
+   if (BLOCK_VER == 1) {
+      ser_uint32(rec->VolSessionId);
+      ser_uint32(rec->VolSessionTime);
+   } else {
+      block->VolSessionId = rec->VolSessionId;
+      block->VolSessionTime = rec->VolSessionTime;
+   }
+   ser_int32(rec->FileIndex);
+   if (rec->remainder > rec->data_len) {
+      ser_int32(rec->Stream);      /* normal full header */
+      ser_uint32(rec->data_len);
+      rec->remainder = rec->data_len; /* must still do data record */
+   } else {
+      ser_int32(-rec->Stream);     /* mark this as a continuation record */
+      ser_uint32(rec->remainder);  /* bytes to do */
+   }
+
+   /* Require enough room to write a full header */
+   ASSERT(rec->remlen >= WRITE_RECHDR_LENGTH);
+
+   block->bufp += WRITE_RECHDR_LENGTH;
+   block->binbuf += WRITE_RECHDR_LENGTH;
+   rec->remlen -= WRITE_RECHDR_LENGTH;
+   if (rec->FileIndex > 0) {
+      /* If data record, update what we have in this block */
+      if (block->FirstIndex == 0) {
+         block->FirstIndex = rec->FileIndex;
+      }
+      block->LastIndex = rec->FileIndex;
+   }
+}
+
+static bool write_data_to_block(DEV_BLOCK *block, DEV_RECORD *rec)
+{
+   rec->remlen = block->buf_len - block->binbuf;
+   /* Write as much of data as possible */
+   if (rec->remlen >= rec->remainder) {
+      memcpy(block->bufp, rec->data+rec->data_len-rec->remainder,
+             rec->remainder);
+      block->bufp += rec->remainder;
+      block->binbuf += rec->remainder;
+   } else {
+      memcpy(block->bufp, rec->data+rec->data_len-rec->remainder,
+             rec->remlen);
+#ifdef xxxxxSMCHECK
+      if (!sm_check_rtn(__FILE__, __LINE__, False)) {
+         /* We damaged a buffer */
+         Dmsg6(0, "Damaged block FI=%s SessId=%d Strm=%s len=%d\n"
+            "rem=%d remainder=%d\n",
+            FI_to_ascii(buf1, rec->FileIndex), rec->VolSessionId,
+            stream_to_ascii(buf2, rec->Stream, rec->FileIndex), rec->data_len,
+            rec->remlen, rec->remainder);
+         Dmsg5(0, "Damaged block: bufp=%x binbuf=%d buf_len=%d rem=%d moved=%d\n",
+            block->bufp, block->binbuf, block->buf_len, block->buf_len-block->binbuf,
+            rec->remlen);
+         Dmsg2(0, "Damaged block: buf=%x binbuffrombuf=%d \n",
+            block->buf, block->bufp-block->buf);
+         Emsg0(M_ABORT, 0, _("Damaged buffer\n"));
+      }
+#endif
+
+      block->bufp += rec->remlen;
+      block->binbuf += rec->remlen;
+      rec->remainder -= rec->remlen;
+      return false;                /* did partial transfer */
+   }
+   return true;
+}
+
+/*
+ * Write a Record to the block
+ *
+ *  Returns: false means the block could not be written to tape/disk.
+ *           true  on success (all bytes written to the block).
+ */
+bool DCR::write_record(DEV_RECORD *rec)
+{
+   while (!write_record_to_block(this, rec)) {
+      Dmsg2(850, "!write_record_to_block data_len=%d rem=%d\n", rec->data_len,
+                 rec->remainder);
+      if (!write_block_to_device()) {
+         Dmsg2(90, "Got write_block_to_dev error on device %s. %s\n",
+            dev->print_name(), dev->bstrerror());
+         return false;
+      }
+   }
+   return true;
+}
 
 /*
  * Write a Record to the block
@@ -288,154 +435,77 @@ void free_record(DEV_RECORD *rec)
  *  non-zero), and 2. The remaining bytes to write may not
  *  all fit into the block.
  */
-bool write_record_to_block(DEV_BLOCK *block, DEV_RECORD *rec)
+bool write_record_to_block(DCR *dcr, DEV_RECORD *rec)
 {
-   ser_declare;
-   uint32_t remlen;
    char buf1[100], buf2[100];
+   DEV_BLOCK *block = dcr->block;
 
-   remlen = block->buf_len - block->binbuf;
+   for ( ;; ) {
+      ASSERT(block->binbuf == (uint32_t)(block->bufp - block->buf));
+      ASSERT(block->buf_len >= block->binbuf);
 
-   ASSERT(block->binbuf == (uint32_t) (block->bufp - block->buf));
-   ASSERT(block->buf_len >= block->binbuf);
+      Dmsg6(890, "write_record_to_block() FI=%s SessId=%d Strm=%s len=%d\n"
+         "rem=%d remainder=%d\n",
+         FI_to_ascii(buf1, rec->FileIndex), rec->VolSessionId,
+         stream_to_ascii(buf2, rec->Stream, rec->FileIndex), rec->data_len,
+         rec->remlen, rec->remainder);
 
-   Dmsg6(890, "write_record_to_block() FI=%s SessId=%d Strm=%s len=%d\n"
-      "rem=%d remainder=%d\n",
-      FI_to_ascii(buf1, rec->FileIndex), rec->VolSessionId,
-      stream_to_ascii(buf2, rec->Stream, rec->FileIndex), rec->data_len,
-      remlen, rec->remainder);
+      switch (rec->state) {
+      case st_none:
+         /* Figure out what to do */
+         rec->state = st_header;
+         continue;              /* go to next state */
 
-   /*
-    * If rec->remainder is non-zero, we have been called a
-    *  second (or subsequent) time to finish writing a record
-    *  that did not previously fit into the block.
-    */
-   if (rec->remainder == 0) {
-      /* Require enough room to write a full header */
-      if (remlen >= WRITE_RECHDR_LENGTH) {
-         ser_begin(block->bufp, WRITE_RECHDR_LENGTH);
-         if (BLOCK_VER == 1) {
-            ser_uint32(rec->VolSessionId);
-            ser_uint32(rec->VolSessionTime);
-         } else {
-            block->VolSessionId = rec->VolSessionId;
-            block->VolSessionTime = rec->VolSessionTime;
+      case st_header:
+         /*
+          * Write header
+          *
+          * If rec->remainder is non-zero, we have been called a
+          *  second (or subsequent) time to finish writing a record
+          *  that did not previously fit into the block.
+          */
+          if (!write_header_to_block(block, rec)) {
+             return false;        /* write block then come back here */
+          }
+          rec->state = st_data;  /* after header, now write data */
+          continue;
+
+      /* Write continuation header */
+      case st_header_cont:
+         write_continue_header_to_block(block, rec);
+         rec->state = st_data;
+         if (rec->remlen == 0) {
+            return false;                   /* partial transfer */
          }
-         ser_int32(rec->FileIndex);
-         ser_int32(rec->Stream);
-         ser_uint32(rec->data_len);
+         continue;
 
-         block->bufp += WRITE_RECHDR_LENGTH;
-         block->binbuf += WRITE_RECHDR_LENGTH;
-         remlen -= WRITE_RECHDR_LENGTH;
-         rec->remainder = rec->data_len;
-         if (rec->FileIndex > 0) {
-            /* If data record, update what we have in this block */
-            if (block->FirstIndex == 0) {
-               block->FirstIndex = rec->FileIndex;
+      /* Write normal data */
+      case st_data:
+         /*
+          * Write data
+          *
+          * Part of it may have already been transferred, and we
+          * may not have enough room to transfer the whole this time.
+          */
+         if (rec->remainder > 0) {
+            if (!write_data_to_block(block, rec)) {
+               rec->state = st_header_cont;
+               return false;
             }
-            block->LastIndex = rec->FileIndex;
          }
-      } else {
-         rec->remainder = rec->data_len + WRITE_RECHDR_LENGTH;
-         return false;
-      }
-   } else {
-      /*
-       * We are here to write unwritten bytes from a previous
-       * time. Presumably we have a new buffer (possibly
-       * containing a volume label), so the new header
-       * should be able to fit in the block -- otherwise we have
-       * an error.  Note, we have to continue splitting the
-       * data record if it is longer than the block.
-       *
-       * First, write the header, then write as much as
-       * possible of the data record.
-       *
-       * Every time we write a header and it is a continuation
-       * of a previous partially written record, we store the
-       * Stream as -Stream in the record header.
-       */
-      ser_begin(block->bufp, WRITE_RECHDR_LENGTH);
-      if (BLOCK_VER == 1) {
-         ser_uint32(rec->VolSessionId);
-         ser_uint32(rec->VolSessionTime);
-      } else {
-         block->VolSessionId = rec->VolSessionId;
-         block->VolSessionTime = rec->VolSessionTime;
-      }
-      ser_int32(rec->FileIndex);
-      if (rec->remainder > rec->data_len) {
-         ser_int32(rec->Stream);      /* normal full header */
-         ser_uint32(rec->data_len);
-         rec->remainder = rec->data_len; /* must still do data record */
-      } else {
-         ser_int32(-rec->Stream);     /* mark this as a continuation record */
-         ser_uint32(rec->remainder);  /* bytes to do */
-      }
+         rec->remainder = 0;                /* did whole transfer */
+         rec->state = st_none;
+         return true;
 
-      /* Require enough room to write a full header */
-      ASSERT(remlen >= WRITE_RECHDR_LENGTH);
-
-      block->bufp += WRITE_RECHDR_LENGTH;
-      block->binbuf += WRITE_RECHDR_LENGTH;
-      remlen -= WRITE_RECHDR_LENGTH;
-      if (rec->FileIndex > 0) {
-         /* If data record, update what we have in this block */
-         if (block->FirstIndex == 0) {
-            block->FirstIndex = rec->FileIndex;
-         }
-         block->LastIndex = rec->FileIndex;
+      default:
+         Dmsg0(000, "Something went wrong. Default state.\n");
+         rec->state = st_none;
+         return true;
       }
    }
-   if (remlen == 0) {
-      return false;                   /* partial transfer */
-   }
-
-   /*
-    * Now deal with data record.
-    * Part of it may have already been transferred, and we
-    * may not have enough room to transfer the whole this time.
-    */
-   if (rec->remainder > 0) {
-      /* Write as much of data as possible */
-      if (remlen >= rec->remainder) {
-         memcpy(block->bufp, rec->data+rec->data_len-rec->remainder,
-                rec->remainder);
-         block->bufp += rec->remainder;
-         block->binbuf += rec->remainder;
-      } else {
-         memcpy(block->bufp, rec->data+rec->data_len-rec->remainder,
-                remlen);
-#ifdef xxxxxSMCHECK
-         if (!sm_check_rtn(__FILE__, __LINE__, False)) {
-            /* We damaged a buffer */
-            Dmsg6(0, "Damaged block FI=%s SessId=%d Strm=%s len=%d\n"
-               "rem=%d remainder=%d\n",
-               FI_to_ascii(buf1, rec->FileIndex), rec->VolSessionId,
-               stream_to_ascii(buf2, rec->Stream, rec->FileIndex), rec->data_len,
-               remlen, rec->remainder);
-            Dmsg5(0, "Damaged block: bufp=%x binbuf=%d buf_len=%d rem=%d moved=%d\n",
-               block->bufp, block->binbuf, block->buf_len, block->buf_len-block->binbuf,
-               remlen);
-            Dmsg2(0, "Damaged block: buf=%x binbuffrombuf=%d \n",
-               block->buf, block->bufp-block->buf);
-
-               Emsg0(M_ABORT, 0, _("Damaged buffer\n"));
-         }
-#endif
-
-         block->bufp += remlen;
-         block->binbuf += remlen;
-         rec->remainder -= remlen;
-         return false;                /* did partial transfer */
-      }
-   }
-   rec->remainder = 0;                /* did whole transfer */
    return true;
 }
 
-
 /*
  * Test if we can write whole record to the block
  *
@@ -476,7 +546,7 @@ uint64_t get_record_address(DEV_RECORD *rec)
  *                 routine may have to be called again with a new
  *                 block if the entire record was not read.
  */
-bool read_record_from_block(DCR *dcr, DEV_BLOCK *block, DEV_RECORD *rec)
+bool read_record_from_block(DCR *dcr, DEV_RECORD *rec)
 {
    ser_declare;
    uint32_t remlen;
@@ -488,45 +558,45 @@ bool read_record_from_block(DCR *dcr, DEV_BLOCK *block, DEV_RECORD *rec)
    uint32_t rhl;
    char buf1[100], buf2[100];
 
-   remlen = block->binbuf;
+   remlen = dcr->block->binbuf;
 
    /* Clear state flags */
-   rec->state = 0;
-   if (block->dev->is_tape()) {
-      rec->state |= REC_ISTAPE;
+   rec->state_bits = 0;
+   if (dcr->block->dev->is_tape()) {
+      rec->state_bits |= REC_ISTAPE;
    }
-   rec->Block = ((DEVICE *)block->dev)->EndBlock;
-   rec->File = ((DEVICE *)block->dev)->EndFile;
+   rec->Block = ((DEVICE *)(dcr->block->dev))->EndBlock;
+   rec->File = ((DEVICE *)(dcr->block->dev))->EndFile;
 
    /*
     * Get the header. There is always a full header,
     * otherwise we find it in the next block.
     */
-   Dmsg3(450, "Block=%d Ver=%d size=%u\n", block->BlockNumber, block->BlockVer,
-         block->block_len);
-   if (block->BlockVer == 1) {
+   Dmsg3(450, "Block=%d Ver=%d size=%u\n", dcr->block->BlockNumber, dcr->block->BlockVer,
+         dcr->block->block_len);
+   if (dcr->block->BlockVer == 1) {
       rhl = RECHDR1_LENGTH;
    } else {
       rhl = RECHDR2_LENGTH;
    }
    if (remlen >= rhl) {
       Dmsg4(450, "Enter read_record_block: remlen=%d data_len=%d rem=%d blkver=%d\n",
-            remlen, rec->data_len, rec->remainder, block->BlockVer);
+            remlen, rec->data_len, rec->remainder, dcr->block->BlockVer);
 
-      unser_begin(block->bufp, WRITE_RECHDR_LENGTH);
-      if (block->BlockVer == 1) {
+      unser_begin(dcr->block->bufp, WRITE_RECHDR_LENGTH);
+      if (dcr->block->BlockVer == 1) {
          unser_uint32(VolSessionId);
          unser_uint32(VolSessionTime);
       } else {
-         VolSessionId = block->VolSessionId;
-         VolSessionTime = block->VolSessionTime;
+         VolSessionId = dcr->block->VolSessionId;
+         VolSessionTime = dcr->block->VolSessionTime;
       }
       unser_int32(FileIndex);
       unser_int32(Stream);
       unser_uint32(data_bytes);
 
-      block->bufp += rhl;
-      block->binbuf -= rhl;
+      dcr->block->bufp += rhl;
+      dcr->block->binbuf -= rhl;
       remlen -= rhl;
 
       /* If we are looking for more (remainder!=0), we reject anything
@@ -534,7 +604,7 @@ bool read_record_from_block(DCR *dcr, DEV_BLOCK *block, DEV_RECORD *rec)
        */
       if (rec->remainder && (rec->VolSessionId != VolSessionId ||
                              rec->VolSessionTime != VolSessionTime)) {
-         rec->state |= REC_NO_MATCH;
+         rec->state_bits |= REC_NO_MATCH;
          Dmsg0(450, "remainder and VolSession doesn't match\n");
          return false;             /* This is from some other Session */
       }
@@ -545,11 +615,11 @@ bool read_record_from_block(DCR *dcr, DEV_BLOCK *block, DEV_RECORD *rec)
       if (Stream < 0) {               /* continuation record? */
          Dmsg1(500, "Got negative Stream => continuation. remainder=%d\n",
             rec->remainder);
-         rec->state |= REC_CONTINUATION;
+         rec->state_bits |= REC_CONTINUATION;
          if (!rec->remainder) {       /* if we didn't read previously */
             rec->data_len = 0;        /* return data as if no continuation */
          } else if (rec->Stream != -Stream) {
-            rec->state |= REC_NO_MATCH;
+            rec->state_bits |= REC_NO_MATCH;
             return false;             /* This is from some other Session */
          }
          rec->Stream = -Stream;       /* set correct Stream */
@@ -563,10 +633,10 @@ bool read_record_from_block(DCR *dcr, DEV_BLOCK *block, DEV_RECORD *rec)
       rec->VolSessionTime = VolSessionTime;
       rec->FileIndex = FileIndex;
       if (FileIndex > 0) {
-         if (block->FirstIndex == 0) {
-            block->FirstIndex = FileIndex;
+         if (dcr->block->FirstIndex == 0) {
+            dcr->block->FirstIndex = FileIndex;
          }
-         block->LastIndex = FileIndex;
+         dcr->block->LastIndex = FileIndex;
       }
 
       Dmsg6(450, "rd_rec_blk() got FI=%s SessId=%d Strm=%s len=%u\n"
@@ -584,8 +654,8 @@ bool read_record_from_block(DCR *dcr, DEV_BLOCK *block, DEV_RECORD *rec)
        * then reread.
        */
       Dmsg0(450, "read_record_block: nothing\n");
-      rec->state |= (REC_NO_HEADER | REC_BLOCK_EMPTY);
-      empty_block(block);                      /* mark block empty */
+      rec->state_bits |= (REC_NO_HEADER | REC_BLOCK_EMPTY);
+      empty_block(dcr->block);                 /* mark block empty */
       return false;
    }
 
@@ -595,8 +665,8 @@ bool read_record_from_block(DCR *dcr, DEV_BLOCK *block, DEV_RECORD *rec)
        * Something is wrong, force read of next block, abort 
        *   continuing with this block.
        */
-      rec->state |= (REC_NO_HEADER | REC_BLOCK_EMPTY);
-      empty_block(block);
+      rec->state_bits |= (REC_NO_HEADER | REC_BLOCK_EMPTY);
+      empty_block(dcr->block);
       Jmsg2(dcr->jcr, M_WARNING, 0, _("Sanity check failed. maxlen=%d datalen=%d. Block discarded.\n"),
          MAX_BLOCK_LENGTH, data_bytes);
       return false;
@@ -614,19 +684,19 @@ bool read_record_from_block(DCR *dcr, DEV_BLOCK *block, DEV_RECORD *rec)
     */
    if (remlen >= data_bytes) {
       /* Got whole record */
-      memcpy(rec->data+rec->data_len, block->bufp, data_bytes);
-      block->bufp += data_bytes;
-      block->binbuf -= data_bytes;
+      memcpy(rec->data+rec->data_len, dcr->block->bufp, data_bytes);
+      dcr->block->bufp += data_bytes;
+      dcr->block->binbuf -= data_bytes;
       rec->data_len += data_bytes;
    } else {
       /* Partial record */
-      memcpy(rec->data+rec->data_len, block->bufp, remlen);
-      block->bufp += remlen;
-      block->binbuf -= remlen;
+      memcpy(rec->data+rec->data_len, dcr->block->bufp, remlen);
+      dcr->block->bufp += remlen;
+      dcr->block->binbuf -= remlen;
       rec->data_len += remlen;
       rec->remainder = 1;             /* partial record transferred */
       Dmsg1(450, "read_record_block: partial xfered=%d\n", rec->data_len);
-      rec->state |= (REC_PARTIAL_RECORD | REC_BLOCK_EMPTY);
+      rec->state_bits |= (REC_PARTIAL_RECORD | REC_BLOCK_EMPTY);
       return true;
    }
    rec->remainder = 0;
