@@ -1,7 +1,7 @@
 /*
    Bacula® - The Network Backup Solution
 
-   Copyright (C) 2001-2009 Free Software Foundation Europe e.V.
+   Copyright (C) 2001-2012 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -62,13 +62,8 @@
      Note, the original W.Z. Venema smtp.c had no license and no
      copyright.  See:
         http://archives.neohapsis.com/archives/postfix/2000-05/1520.html
- 
-
-   Version $Id$
-
  */
 
-
 #include "bacula.h"
 #include "jcr.h"
 #define MY_NAME "bsmtp"
@@ -77,14 +72,24 @@
 #include <lmcons.h>
 #endif
 
-/* Dummy functions */
+/*
+ * Dummy functions
+ */
 int generate_daemon_event(JCR *jcr, const char *event) 
-   { return 1; }
+{
+   return 1;
+}
 
 #ifndef MAXSTRING
 #define MAXSTRING 254
 #endif
 
+enum resolv_type {
+   RESOLV_PROTO_ANY,
+   RESOLV_PROTO_IPV4,
+   RESOLV_PROTO_IPV6
+};
+
 static FILE *sfp;
 static FILE *rfp;
 
@@ -97,6 +102,7 @@ static char *reply_addr = NULL;
 static int mailport = 25;
 static char my_hostname[MAXSTRING];
 static bool content_utf8 = false;
+static resolv_type default_resolv_type = RESOLV_PROTO_IPV4;
 
 /* 
  * Take input that may have names and other stuff and strip
@@ -185,7 +191,12 @@ static void usage()
    fprintf(stderr,
 _("\n"
 "Usage: %s [-f from] [-h mailhost] [-s subject] [-c copy] [recipient ...]\n"
+"       -4          forces bsmtp to use IPv4 addresses only.\n"
+#ifdef HAVE_IPV6
+"       -6          forces bsmtp to use IPv6 addresses only.\n"
+#endif
 "       -8          set charset to UTF-8\n"
+"       -a          use any ip protocol for address resolution\n"
 "       -c          set the Cc: field\n"
 "       -d <nn>     set debug level to <nn>\n"
 "       -dt         print a timestamp in debug output\n"
@@ -202,10 +213,9 @@ _("\n"
 
 /*
  * Return the offset west from localtime to UTC in minutes
-  * Same as timezone.tz_minuteswest
-  *   Unix tz_offset coded by:  Attila Fülöp
-  */
-
+ * Same as timezone.tz_minuteswest
+ *   Unix tz_offset coded by:  Attila Fülöp
+ */
 static long tz_offset(time_t lnow, struct tm &tm)
 {
 #if defined(HAVE_WIN32)
@@ -247,31 +257,42 @@ static void get_date_string(char *buf, int buf_len)
 
    my_timezone = tz_offset(now, tm);
    strftime(buf, buf_len, "%a, %d %b %Y %H:%M:%S", &tm);
-   sprintf(tzbuf, " %+2.2ld%2.2u", -my_timezone / 60, abs(my_timezone) % 60);
+   snprintf(tzbuf, sizeof(tzbuf), " %+2.2ld%2.2u", -my_timezone / 60, abs(my_timezone) % 60);
    strcat(buf, tzbuf);              /* add +0100 */
    strftime(tzbuf, sizeof(tzbuf), " (%Z)", &tm);
    strcat(buf, tzbuf);              /* add (CEST) */
 }
 
-
 /*********************************************************************
  *
  *  Program to send email
  */
 int main (int argc, char *argv[])
 {
-    char buf[1000];
-    struct sockaddr_in sin;
-    struct hostent *hp;
-    int i, ch;
-    unsigned long maxlines, lines;
+   char buf[1000];
+   int i, ch;
+   unsigned long maxlines, lines;
 #if defined(HAVE_WIN32)
-    SOCKET s;
+   SOCKET s;
 #else
-    int s, r;
-    struct passwd *pwd;
+   int s, r;
+   struct passwd *pwd;
+#endif
+   char *cp, *p;
+#ifdef HAVE_GETADDRINFO
+   int res;
+   struct addrinfo hints;
+   struct addrinfo *ai, *rp;
+   char mail_port[10];
+#else
+   struct hostent *hp;
+   struct sockaddr_in sin;
+#endif
+#ifdef HAVE_IPV6
+   const char *options = "468ac:d:f:h:r:s:l:?";
+#else
+   const char *options = "48ac:d:f:h:r:s:l:?";
 #endif
-    char *cp, *p;
     
    setlocale(LC_ALL, "en_US");
    bindtextdomain("bacula", LOCALEDIR);
@@ -280,11 +301,26 @@ int main (int argc, char *argv[])
    my_name_is(argc, argv, "bsmtp");
    maxlines = 0;
 
-   while ((ch = getopt(argc, argv, "8c:d:f:h:r:s:l:?")) != -1) {
+   while ((ch = getopt(argc, argv, options)) != -1) {
       switch (ch) {
+      case '4':
+         default_resolv_type = RESOLV_PROTO_IPV4;
+         break;
+
+#ifdef HAVE_IPV6
+      case '6':
+         default_resolv_type = RESOLV_PROTO_IPV6;
+         break;
+#endif
+
       case '8':
          content_utf8 = true;
          break;
+
+      case 'a':
+         default_resolv_type = RESOLV_PROTO_ANY;
+         break;
+
       case 'c':
          Dmsg1(20, "cc=%s\n", optarg);
          cc_addr = optarg;
@@ -345,7 +381,6 @@ int main (int argc, char *argv[])
       exit(1);
    }
 
-
    /*
     *  Determine SMTP server
     */
@@ -372,12 +407,28 @@ int main (int argc, char *argv[])
       Pmsg1(0, _("Fatal gethostname error: ERR=%s\n"), strerror(errno));
       exit(1);
    }
+#ifdef HAVE_GETADDRINFO
+   memset(&hints, 0, sizeof(struct addrinfo));
+   hints.ai_family = AF_UNSPEC;
+   hints.ai_socktype = 0;
+   hints.ai_protocol = 0;
+   hints.ai_flags = AI_CANONNAME;
+
+   if ((res = getaddrinfo(my_hostname, NULL, &hints, &ai)) != 0) {
+      Pmsg2(0, _("Fatal getaddrinfo for myself failed \"%s\": ERR=%s\n"),
+            my_hostname, gai_strerror(res));
+      exit(1);
+   }
+   strcpy(my_hostname, ai->ai_canonname);
+   freeaddrinfo(ai);
+#else
    if ((hp = gethostbyname(my_hostname)) == NULL) {
-      Pmsg2(0, _("Fatal gethostbyname for myself failed \"%s\": ERR=%s\n"), my_hostname,
-         strerror(errno));
+      Pmsg2(0, _("Fatal gethostbyname for myself failed \"%s\": ERR=%s\n"),
+            my_hostname, strerror(errno));
       exit(1);
    }
    strcpy(my_hostname, hp->h_name);
+#endif
    Dmsg1(20, "My hostname is: %s\n", my_hostname);
 
    /*
@@ -389,15 +440,15 @@ int main (int argc, char *argv[])
       LPSTR lpszBuffer = (LPSTR)alloca(dwSize);
 
       if (GetUserName(lpszBuffer, &dwSize)) {
-         sprintf(buf, "%s@%s", lpszBuffer, my_hostname);
+         snprintf(buf, sizeof(buf), "%s@%s", lpszBuffer, my_hostname);
       } else {
-         sprintf(buf, "unknown-user@%s", my_hostname);
+         snprintf(buf, sizeof(buf), "unknown-user@%s", my_hostname);
       }
 #else
       if ((pwd = getpwuid(getuid())) == 0) {
-         sprintf(buf, "userid-%d@%s", (int)getuid(), my_hostname);
+         snprintf(buf, sizeof(buf), "userid-%d@%s", (int)getuid(), my_hostname);
       } else {
-         sprintf(buf, "%s@%s", pwd->pw_name, my_hostname);
+         snprintf(buf, sizeof(buf), "%s@%s", pwd->pw_name, my_hostname);
       }
 #endif
       from_addr = bstrdup(buf);
@@ -407,14 +458,72 @@ int main (int argc, char *argv[])
    /*
     *  Connect to smtp daemon on mailhost.
     */
-hp:
+lookup_host:
+#ifdef HAVE_GETADDRINFO
+   memset(&hints, 0, sizeof(struct addrinfo));
+   switch (default_resolv_type) {
+   case RESOLV_PROTO_ANY:
+      hints.ai_family = AF_UNSPEC;
+      break;
+   case RESOLV_PROTO_IPV4:
+      hints.ai_family = AF_INET;
+      break;
+#ifdef HAVE_IPV6
+   case RESOLV_PROTO_IPV6:
+      hints.ai_family = AF_INET6;
+      break;
+#endif
+   default:
+      hints.ai_family = AF_UNSPEC;
+      break;
+   }
+   hints.ai_socktype = SOCK_STREAM;
+   hints.ai_protocol = 0;
+   hints.ai_flags = 0;
+   snprintf(mail_port, sizeof(mail_port), "%d", mailport);
+
+   if ((res = getaddrinfo(mailhost, mail_port, &hints, &ai)) != 0) {
+      Pmsg2(0, _("Error unknown mail host \"%s\": ERR=%s\n"),
+            mailhost, gai_strerror(res));
+      if (strcasecmp(mailhost, "localhost")) {
+         Pmsg0(0, _("Retrying connection using \"localhost\".\n"));
+         mailhost = "localhost";
+         goto lookup_host;
+      }
+      exit(1);
+   }
+
+   for (rp = ai; rp != NULL; rp = rp->ai_next) {
+#if defined(HAVE_WIN32)
+      s = WSASocket(rp->ai_family, rp->ai_socktype, rp->ai_protocol, NULL, 0, 0);
+#else
+      s = socket(rp->ai_family, rp->ai_socktype, rp->ai_protocol);
+#endif
+      if (s < 0) {
+         continue;
+      }
+
+      if (connect(s, rp->ai_addr, rp->ai_addrlen) != -1) {
+         break;
+      }
+
+      close(s);
+   }
+
+   if (!rp) {
+      Pmsg1(0, _("Failed to connect to mailhost %s\n"), mailhost);
+      exit(1);
+   }
+
+   freeaddrinfo(ai);
+#else
    if ((hp = gethostbyname(mailhost)) == NULL) {
       Pmsg2(0, _("Error unknown mail host \"%s\": ERR=%s\n"), mailhost,
          strerror(errno));
       if (strcasecmp(mailhost, "localhost") != 0) {
          Pmsg0(0, _("Retrying connection using \"localhost\".\n"));
          mailhost = "localhost";
-         goto hp;
+         goto lookup_host;
       }
       exit(1);
    }
@@ -443,6 +552,7 @@ hp:
       exit(1);
    }
    Dmsg0(20, "Connected\n");
+#endif
 
 #if defined(HAVE_WIN32)
    int fdSocket = _open_osfhandle(s, _O_RDWR | _O_BINARY);
