@@ -1,8 +1,8 @@
 
 #undef  VERSION
-#define VERSION "5.2.6"
-#define BDATE   "21 February 2012"
-#define LSMDATE "21Feb12"
+#define VERSION "5.2.12"
+#define BDATE   "12 September 2012"
+#define LSMDATE "12Sep12"
 
 #define PROG_COPYRIGHT "Copyright (C) %d-2012 Free Software Foundation Europe e.V.\n"
 #define BYEAR "2012"       /* year for copyright messages in progs */
@@ -77,7 +77,6 @@
  *   it can always be turned on, but we advise to use it only
  *   for debug
  */
-#if DEVELOPER
 # ifndef _USE_LOCKMGR
 #  define _USE_LOCKMGR
 # endif /* _USE_LOCKMGR */
@@ -98,7 +97,6 @@
  * dozens of thread, so turn this only for debugging.
  */
 /* #define USE_LOCKMGR_SAFEKILL */
-#endif  /* DEVELOPER */
 
 #if !HAVE_LINUX_OS && !HAVE_SUN_OS && !HAVE_DARWIN_OS && !HAVE_FREEBSD_OS
 # undef _USE_LOCKMGR
@@ -146,6 +144,7 @@
 /* #define TRACE_RES 1 */
 /* #define DEBUG_MEMSET 1 */
 /* #define DEBUG_MUTEX 1 */
+/* #define DEBUG_BLOCK_CHECKSUM 1 */
 #define BEEF 0
 
 /*
